import { createSlice, createAsyncThunk, PayloadAction } from "@reduxjs/toolkit";
import http from "../../lib/https";
import { toast } from "react-toastify";

// Interfaces
interface User {
	token: string | null;
	id: string;
	name: string;
	email: string;
	role: string;
	phone: string;
}

interface Admin {
	id: string;
	name: string;
	role: string;
	email: string;
	phone: string;
}

interface AuthState {
	user: User | null;
	admins: Admin[];
	isAuthenticated: boolean;
	isLoading: boolean;
	error: string | null;
	token: string | null;
}

// Fetch Admin List
export const adminList = createAsyncThunk<
	Admin[],
	void,
	{ rejectValue: string }
>("FetchAdminList", async (_, { rejectWithValue }) => {
	try {
		const response = await http.get("auth/admin-list");
		return response?.data?.data;
	} catch (error: any) {
		return rejectWithValue(
			error.response?.data?.message || "An error occurred"
		);
	}
});

// Delete Admin
export const deleteAdmin = createAsyncThunk<
	string,
	string,
	{ rejectValue: string }
>("deleteAdmin", async (id, { rejectWithValue }) => {
	try {
		const response = await http.delete(`auth/${id}/delete-admin`);
		toast.success(response.data?.message);
		return id;
	} catch (error: any) {
		return rejectWithValue(
			error.response?.data?.message || "An error occurred"
		);
	}
});

// Update Admin
export const updateAdmin = createAsyncThunk(
	"UpdateAdmin",
	async (
		{ id, name, role }: { id: any; name: string; role: string },
		{ rejectWithValue }
	) => {
		try {
			const response = await http.put(`auth/${id}/update-admin`, {
				name,
				role,
			});
			toast.success("Admin updated successfully");
			return response?.data;
		} catch (error: any) {
			return rejectWithValue(
				error.response?.data?.message || "An error occurred"
			);
		}
	}
);

const initialState: AuthState = {
	user: null,
	admins: [],
	isAuthenticated: false,
	isLoading: false,
	error: null,
	token: null,
};

const adminSlice = createSlice({
	name: "admin",
	initialState,
	reducers: {
	},
	extraReducers: (builder) => {
		builder
			.addCase(adminList.pending, (state) => {
				state.isLoading = true;
				state.error = null;
			})
			.addCase(
				adminList.fulfilled,
				(state, action: PayloadAction<Admin[]>) => {
					state.isLoading = false;
					state.admins = action.payload;
				}
			)
			.addCase(
				adminList.rejected,
				(state, action: PayloadAction<string | undefined>) => {
					state.isLoading = false;
					state.error = action.payload || "An error occurred";
				}
			)
			.addCase(deleteAdmin.pending, (state) => {
				state.isLoading = true;
			})
			.addCase(deleteAdmin.fulfilled, (state, action) => {
				state.isLoading = false;
				state.admins = state.admins.filter(
					(admin) => String(admin.id) !== String(action.payload)
				);
			})
			.addCase(deleteAdmin.rejected, (state, action) => {
				state.isLoading = false;
				state.error = action.payload || "Failed to delete admin";
			})
			.addCase(updateAdmin.pending, (state) => {
				state.isLoading = true;
				state.error = null;
			})
			.addCase(updateAdmin.fulfilled, (state, action) => {
				const updatedAdmin = action.payload;
				state.admins = state?.admins?.map((admin) =>
					admin?.id === updatedAdmin?.id ? updatedAdmin : admin
				);
				state.isLoading = false;
			})
			.addCase(updateAdmin.rejected, (state, action) => {
				state.isLoading = false;
				state.error =
					typeof action.payload === "string"
						? action.payload
						: "Something went wrong while updating Admin!";
			});
	},
});

export default adminSlice.reducer;
