import { createSlice, createAsyncThunk } from "@reduxjs/toolkit";
import http from "../../lib/https";

interface User {
	token: string | null;
	id: string;
	name: string;
	email: string;
	role: string;
	phone: string;
}

interface AuthState {
	user: User | null;
	isAuthenticated: boolean;
	isLoading: boolean;
	error: string | null;
}

export const fetchAdminProfile = createAsyncThunk<
	User,
	void,
	{ rejectValue: string }
>("GetAdminProfile", async (_, { rejectWithValue }) => {
	try {
		const token = localStorage?.getItem("authToken");
		if (!token) throw new Error("No token found");

		const response = await http.get("/auth/profile", {
			headers: { Authorization: `Bearer ${token}` },
		});

		if (!response.data?.data) throw new Error("Invalid API response");

		return response.data.data;
	} catch (error: any) {
		return rejectWithValue(error?.response?.data?.message || "An error occurred");
	}
});

const initialState: AuthState = {
	user: null,
	isAuthenticated: false,
	isLoading: false,
	error: null,
};

const profileSlice = createSlice({
	name: "profile",
	initialState,
	reducers: {
	},
	extraReducers: (builder) => {
		builder
			.addCase(fetchAdminProfile.pending, (state) => {
				state.isLoading = true;
				state.error = null;
			})
			.addCase(fetchAdminProfile.fulfilled, (state, action) => {
				state.isLoading = false;
				state.user = action.payload;
				state.isAuthenticated = true;
			})
			.addCase(fetchAdminProfile.rejected, (state, action) => {
				state.isLoading = false;
				state.error = action.payload || "Failed to fetch admin profile";
			});
	},
});

export default profileSlice.reducer;
